@@ -4,17 +4,22 @@ use strict;
 
 use DateTime::LocaleCatalog;
 
+# Loading this here isn't necessary, but it makes it easier to catch
+# syntax errors when testing.
+use DateTime::Locale::Base;
+
 use Params::Validate qw( validate validate_pos SCALAR );
 
 use vars qw($VERSION);
 
-$VERSION = 0.09;
+$VERSION = 0.3101;
 
 BEGIN
 {
-    return unless $] >= 5.006;
-
-    require utf8; import utf8;
+    if ( $] >= 5.006 )
+    {
+        require utf8; import utf8;
+    }
 }
 
 my %Class;
@@ -50,14 +55,16 @@ sub _register
                       { id               => { type => SCALAR },
 
                         en_language      => { type => SCALAR },
+                        en_script        => { type => SCALAR, optional => 1 },
                         en_territory     => { type => SCALAR, optional => 1 },
                         en_variant       => { type => SCALAR, optional => 1 },
 
                         native_language  => { type => SCALAR, optional => 1 },
+                        native_script    => { type => SCALAR, optional => 1 },
                         native_territory => { type => SCALAR, optional => 1 },
                         native_variant   => { type => SCALAR, optional => 1 },
                         # undocumented hack so we don't have to
-                        # generate .pm files for ICU XML locales which
+                        # generate .pm files for CLDR XML locales which
                         # don't differ from their parents in terms of
                         # datetime data.
                         real_class       => { type => SCALAR, optional => 1 },
@@ -78,7 +85,7 @@ sub _register
 
     my @en_pieces;
     my @native_pieces;
-    foreach my $p ( qw( language territory variant ) )
+    foreach my $p ( qw( language script territory variant ) )
     {
         push @en_pieces, $p{"en_$p"} if exists $p{"en_$p"};
         push @native_pieces, $p{"native_$p"} if exists $p{"native_$p"};
@@ -162,28 +169,28 @@ sub native_names     { wantarray ? keys %NativeNameToID  : [ keys %NativeNameToI
 # These are hardcoded for backwards comaptibility with the
 # DateTime::Language code.
 my %OldAliases =
-    ( #'Afar'              => undef, # XXX
-     'Amharic'           => 'am_ET',
-     'Austrian'          => 'de_AT',
-     'Brazilian'         => 'pt_BR',
-     'Czech'             => 'cs_CZ',
-     'Danish'            => 'da_DK',
-     'Dutch'             => 'nl_NL',
-     'English'           => 'en_US',
-     'French'            => 'fr_FR',
-     #      'Gedeo'             => undef, # XXX
-     'German'            => 'de_DE',
-     'Italian'           => 'it_IT',
-     'Norwegian'         => 'no_NO',
-     'Oromo'             => 'om_ET', # Maybe om_KE or plain om ?
-     'Portugese'         => 'pt_PT',
-     #      'Sidama'            => undef, # XXX
-     'Somali'            => 'so_SO',
-     'Spanish'           => 'es_ES',
-     'Swedish'           => 'sv_SE',
-     #      'Tigre'             => undef, # XXX
-     'TigrinyaEthiopian' => 'ti_ET',
-     'TigrinyaEritrean'  => 'ti_ER',
+    ( 'Afar'             => 'aa',
+      'Amharic'           => 'am_ET',
+      'Austrian'          => 'de_AT',
+      'Brazilian'         => 'pt_BR',
+      'Czech'             => 'cs_CZ',
+      'Danish'            => 'da_DK',
+      'Dutch'             => 'nl_NL',
+      'English'           => 'en_US',
+      'French'            => 'fr_FR',
+      #      'Gedeo'             => undef, # XXX
+      'German'            => 'de_DE',
+      'Italian'           => 'it_IT',
+      'Norwegian'         => 'no_NO',
+      'Oromo'             => 'om_ET', # Maybe om_KE or plain om ?
+      'Portugese'         => 'pt_PT',
+      'Sidama'            => 'sid',
+      'Somali'            => 'so_SO',
+      'Spanish'           => 'es_ES',
+      'Swedish'           => 'sv_SE',
+      'Tigre'             => 'tig',
+      'TigrinyaEthiopian' => 'ti_ET',
+      'TigrinyaEritrean'  => 'ti_ER',
     );
 
 sub load
@@ -191,6 +198,9 @@ sub load
     my $class = shift;
     my $name = shift;
 
+    # Support RFC 3066 language tags, which use '-' instead of '_'.
+    $name =~ tr/-/_/;
+
     my $key = $name;
 
     return $LoadCache{$key} if exists $LoadCache{$key};
@@ -198,7 +208,7 @@ sub load
     # Custom class registered by user
     if ( $Class{$name} )
     {
-        return $LoadCache{$key} = $Class{$name}->new;
+        return $LoadCache{$key} = $class->_load_class_from_id( $name, $Class{$name} )
     }
 
     # special case for backwards compatibility with DT::Language
@@ -231,9 +241,23 @@ sub _guess_id
     # Strip off charset for LC_* ids : en_GB.UTF-8 etc
     $name =~ s/\..*$//;
 
-    my ( $language, $territory, $variant ) = split /_/, $name;
+    my ($language, $script, $territory, $variant ) =
+        parse_id($name);
 
     my @guesses;
+
+    if ( defined $script )
+    {
+        my $guess = join '_', lc $language, ucfirst lc $script;
+
+        push @guesses, $guess;
+
+        $guess .= '_' . uc $territory if defined $territory;
+
+        # version with script comes first
+        unshift @guesses, $guess;
+    }
+
     if ( defined $variant )
     {
         push @guesses,
@@ -255,10 +279,22 @@ sub _guess_id
     }
 }
 
+sub parse_id
+{
+    $_[0] =~ /([a-z]+)               # id
+              (?: _([A-Z][a-z]+) )?  # script - Title Case - optional
+              (?: _([A-Z]+) )?       # territory - ALL CAPS - optional
+              (?: _([A-Z]+) )?       # variant - ALL CAPS - optional
+             /x;
+
+    return $1, $2, $3, $4;
+}
+
 sub _load_class_from_id
 {
     my $class = shift;
     my $id = shift;
+    my $real_class = shift;
 
     # We want the first alias for which there is data, even if it has
     # no corresponding .pm file.  There may be multiple levels of
@@ -272,7 +308,7 @@ sub _load_class_from_id
     my $data = $DataForID{$data_id};
     my $subclass = $data->{real_class} ? $data->{real_class} : $data_id;
 
-    my $real_class = "DateTime::Locale::$subclass";
+    $real_class ||= "DateTime::Locale::$subclass";
 
     unless ( $real_class->can('new') )
     {
@@ -292,7 +328,7 @@ __END__
 
 =head1 NAME
 
-DateTime::Locale - Localization support for DateTime
+DateTime::Locale - Localization support for DateTime.pm
 
 =head1 SYNOPSIS
 
@@ -306,7 +342,7 @@ DateTime::Locale - Localization support for DateTime
   # but mostly just things like ...
 
   my $dt = DateTime->now( locale => 'fr' );
-  print "Aujord'hui le mois est " . $dt->month_name, "\n":
+  print "Aujourd'hui le mois est " . $dt->month_name, "\n":
 
 =head1 DESCRIPTION
 
@@ -335,6 +371,8 @@ find a suitable replacement.
 
 The fallback search order is:
 
+  language_script_territory
+  language_script
   language_territory_variant
   language_territory
   language
@@ -346,6 +384,14 @@ Eg. For locale C<es_XX_UNKNOWN> the fallback search would be:
   es              # Found - the es locale is returned as the
                   # closest match to the requested id
 
+Eg. For locale C<es_Latn_XX> the fallback search would be:
+
+  es_Latn_XX      # Fails - no such locale
+  es_Latn         # Fails - no such locale
+  es_XX           # Fails - no such locale
+  es              # Found - the es locale is returned as the
+                  # closest match to the requested id
+
 If no suitable replacement is found, then an exception is thrown.
 
 Please note that if you provide an B<id> to this method, then the
@@ -440,10 +486,12 @@ C<names()>.
            en_language      => ..., # something like 'English' or 'Afar',
 
            # All other keys are optional.  These are:
+           en_script    => ...,
            en_territory => ...,
            en_variant   => ...,
 
            native_language  => ...,
+           native_sript     => ...,
            native_territory => ...,
            native_variant   => ...,
 
@@ -456,10 +504,12 @@ C<names()>.
 The locale id and English name are required, and the following formats
 should used wherever possible:
 
- id:   languageId[_territoryId[_variantId]]
+ id:   languageId[_script][_territoryId[_variantId]]
 
- Where:  languageId = Lower case ISO  639 code -
-          Always choose 639-1 over 639-2 where possible.
+ Where:  languageId = Lower case ISO 639 code -
+         Always choose 639-1 over 639-2 where possible.
+
+ script = Title Case ISO 15924 script code
 
  territoryId = Upper case ISO 3166 code -
                Always choose 3166-1 over 3166-2 where possible.
@@ -515,8 +565,8 @@ Examples:
  # NOT Ridas::Locales::Custom::en_GB_RIDAS !
  my $l = DateTime::Locale->load('en_GB_RIDAS');
 
-If you a locale for that id already exists, you must specify the
-"replace" parameter as true, or an exception will be thrown.
+If you register a locale for an id that already exists, the "replace"
+parameter must be true or an exception will be thrown.
 
 The complete name for a registered locale is generated by joining
 together the language, territory, and variant components with a single
@@ -564,23 +614,30 @@ provide different date/time formats:
   my $locale_id = 'en_GB_RIDAS1';
 
   my $date_formats =
-  [
-    "%A %{day} %B %{ce_year}",
-    "%{day} %B %{ce_year}",
-    "%{day} %b %{ce_year}",
-    "%{day}/%m/%y",
-  ];
+  {
+    'full'   => '%A %{day} %B %{ce_year}',
+    'long'   => '%{day} %B %{ce_year}',
+    'medium' => '%{day} %b %{ce_year}',
+    'short'  => '%{day}/%m/%y',
+  };
 
   my $time_formats =
-  [
-    "%H h  %{minute} %{time_zone_short_name}",
-    "%{hour12}:%M:%S %p",
-    "%{hour12}:%M:%S %p",
-    "%{hour12}:%M %p",
-  ];
+  {
+    'full'   => '%H h  %{minute} %{time_zone_short_name}',
+    'long'   => '%{hour12}:%M:%S %p',
+    'medium' => '%{hour12}:%M:%S %p',
+    'short'  => '%{hour12}:%M %p',
+  };
 
-  sub date_formats { $date_formats }
-  sub time_formats { $time_formats }
+  sub short_date_format  { $date_formats{short} }
+  sub medium_date_format { $date_formats{medium} }
+  sub long_date_format   { $date_formats{long} }
+  sub full_date_format   { $date_formats{full} }
+
+  sub short_time_format  { $time_formats{short} }
+  sub medium_time_format { $time_formats{medium} }
+  sub long_time_format   { $time_formats{long} }
+  sub full_time_format   { $time_formats{full} }
 
   1;
 
@@ -604,8 +661,17 @@ A completely new custom locale must implement the following methods:
   day_abbreviations
   am_pms
   eras
-  date_formats
-  time_formats
+
+  short_date_format
+  medium_date_format
+  long_date_format
+  full_date_format
+
+  short_time_format
+  medium_time_format
+  long_time_format
+  full_time_format
+
   datetime_format_pattern_order
   date_parts_order
   _default_date_format_length
@@ -642,6 +708,10 @@ The complete locale id, something like "en_US".
 
 The language portion of the id, like "en".
 
+=item * script_id
+
+The script portion of the id, like "Hant".
+
 =item * territory_id
 
 The territory portion of the id, like "US".
@@ -659,6 +729,8 @@ English.
 
 =item * language
 
+=item * script
+
 =item * territory
 
 =item * variant
@@ -672,6 +744,8 @@ The locale's complete name in localized form as a UTF-8 string.
 
 =item * native_language
 
+=item * native_script
+
 =item * native_territory
 
 =item * native_variant
@@ -740,15 +814,15 @@ The following methods deal with the default format lengths:
 These methods return one of "full", "long", "medium", or "short",
 indicating the current default format length.
 
-The default when an object is created is determined by the ICU locale
+The default when an object is created is determined by the CLDR locale
 data.
 
 =item set_default_date_format_length ($length)
 
 =item set_default_time_format_length ($length)
 
-These methods return one of "full", "long", "medium", or "short",
-indicating the current default format length.
+These methods accept one of "full", "long", "medium", or "short",
+indicating the new default format length.
 
 =back
 
@@ -768,6 +842,12 @@ with January as the first month.
 Returns an array reference containing the abbreviated names of the
 months, with January as the first month.
 
+=item * month_narrows
+
+Returns an array reference containing the narrow names of the months,
+with January as the first month.  Narrow names are the shortest
+possible names, and may not be unique.
+
 =item * day_names
 
 Returns an array reference containing the full names of the days,
@@ -778,11 +858,22 @@ with Monday as the first day.
 Returns an array reference containing the abbreviated names of the
 days, with Monday as the first day.
 
+=item * day_narrows
+
+Returns an array reference containing the narrow names of the days,
+with Monday as the first day.  Narrow names are the shortest possible
+names, and may not be unique.
+
 =item * am_pms
 
 Returns an array reference containing the localized forms of "AM" and
 "PM".
 
+=item * eras
+
+Returns an array reference containing the localized forms of "BCE" and
+"CE".
+
 =item * date_formats
 
 Returns a hash reference containing the date formats used for the
@@ -811,19 +902,17 @@ and "ymd".
 
 =head1 SUPPORT
 
-Please be aware that all locale data has been generated from the
-Common XML Locale Repository project locales (originally ICU locale
-data).  The data B<is> currently incomplete, and B<will> contain
-errors in some locales.
+Please be aware that all locale data has been generated from the CLDR
+(Common Locale Data Repository) project locales data).  The data B<is>
+currently incomplete, and B<will> contain errors in some locales.
 
 When reporting errors in data, please check the primary data sources
 first, then where necessary report errors directly to the primary
-source via the ICU project's Jitterbug system at
-http://www.jtcsv.com/cgibin/icu-bugs/
+source via the CLDR bug report system. See
+http://unicode.org/cldr/filing_bug_reports.html for details.
 
-Once these errors have been confirmed, please forward the error
-report, and corrections to the DateTime mailing list,
-datetime@perl.org.
+Once these errors have been confirmed, please forward the error report
+and corrections to the DateTime mailing list, datetime@perl.org.
 
 Support for this module is provided via the datetime@perl.org email
 list. See http://lists.perl.org/ for more details.
@@ -834,27 +923,28 @@ Richard Evans <rich@ridas.com>
 
 Dave Rolsky <autarch@urth.org>
 
-These modules are based on the DateTime::Language modules, which were
-in turn based on the Date::Language modules from Graham Barr's
-TimeDate distribution.
+These modules are loosely based on the DateTime::Language modules,
+which were in turn based on the Date::Language modules from Graham
+Barr's TimeDate distribution.
 
 Thanks to Rick Measham for providing the Java to strftime pattern
 conversion routines used during locale generation.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2003 Richard Evans. All rights reserved.
+Copyright (c) 2003 Richard Evans. Copyright (c) 2004-2006 David
+Rolsky. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
-The full text of the license can be found in the LICENSE file included
+The full text of the license can be found in the F<LICENSE> file included
 with this module.
 
 The locale modules in directory C<DateTime/Locale/> have been
-generated from data provided by the Common XML Locale Repository
-project, see C<DateTime/Locale/LICENSE.icu> for details on the ICU
-data's license.
+generated from data provided by the CLDR project, see
+C<DateTime/Locale/LICENSE.cldr> for details on the CLDR data's
+license.
 
 =head1 SEE ALSO
 
